# SynergyFinder
# Functions for processing drug response matrix.
#
# Functions in this page:
#
# PlotDoseResponse: Visualize the drug combination dose-response data

#' Visualize the drug combination dose-response data
#'
#' A function to visualize the drug combination dose-response data
#'
#' @param data a list object generated by function \code{\link{ReshapeData}}.
#' @param adjusted a logical value. If it is \code{FALSE}, original response 
#'   matrix will be plotted. If it is \code{TRUE}, adjusted response matrix will
#'   be plotted.
#' @param save.file a parameter to specify if the visualization results are 
#'   saved as pdf files in current working directory or not. If it is FALSE, 
#'   the results are returned as a list of the plots. It is FALSE by default.
#' @param file.name a character vector. It indicates the file names, if 
#'   user chose to save the plot to local directory.If it is not defined by
#'   user, a default name will be assigned.
#' @param title a character vector. It indicates the title of each blocks
#'   shown on the plot. If it is not defined by user, a default title with
#'   block IDs will be assigned.
#' @param pair.index a parameter to specify which drug combination if there are
#'   many drug combinations in the data. By default, it is NULL so that the
#'   visualization of all the drug combinations in the data is returned.
#' @param ... further graphical parameters from \code{\link{plot}} for plotting 
#'   the single drug dose-response curve. Use e.g., cex.lab to change the axis 
#'   label size and cex.axis to change the tick size of axises. 
#' @param file.type a character. It indicates the format of files you want to 
#'   save as. Default is "pdf". Available values are "jpeg", "bmp", "png", 
#'   "tiff", "pdf".
#' @return NULL
#'   
#' @author 
#'   \itemize{
#'     \item Liye He \email{liye.he@helsinki.fi},
#'     \item Shuyu Zheng \email{shuyu.zheng@helsinki.fi}
#'   }
#' @import ggplot2
#' @export
#' 
#' @examples
#' data("mathews_screening_data")
#' data <- ReshapeData(mathews_screening_data)
#' PlotDoseResponse(data)
PlotDoseResponse <- function (data, adjusted=TRUE, title=NULL, pair.index=NULL,
                              save.file=FALSE, file.type="pdf",
                              file.name=NULL,  ...) {
  # 1. Check the input data
  if (!is.list(data)) {
    stop("Input data is not a list format!")
  }
  if (!all(c("drug.pairs", "dose.response.mats") %in% names(data))) {
    stop("Input data should contain at least tow elements: 'drug.pairs' and 
         'dose.response.mats'. Please prepare your data with 'ReshapeData' 
         function.")
  }
  if (adjusted & !("adjusted.response.mats" %in% names(data))) {
    stop("'adjusted.response.mats' element is required in input data, when
         argument 'adjusted' is setting as TRUE.")
  }
  # 2. Set the response.mats according to 'adjusted' argument value.
  if (adjusted) {
    response.mats <- data$adjusted.response.mats
  } else {
    response.mats <- data$dose.response.mats
  }

  drug.pairs <- data$drug.pairs

  if(!is.null(pair.index)) {
    blocks <- pair.index
  } else {
    blocks <- names(response.mats)
  }
  
  plots <- vector(mode="list", length=length(blocks))
  names(plots) <- blocks
  
  i <- 1
  for (block in blocks) {
    response.mat <- response.mats[[block]]
    
    # reshape data
    data.plot <- reshape2::melt(response.mat)
    colnames(data.plot) <- c("conc_r", "conc_c", "Inhibition")
    data.plot$conc_r <- as.factor(data.plot$conc_r)
    data.plot$conc_c <- as.factor(data.plot$conc_c)
    data.plot$Inhibition <- signif(c(response.mat), 3)
    conc.runit <- drug.pairs$conc_r_unit[drug.pairs$block_id == block]
    conc.cunit <- drug.pairs$conc_c_unit[drug.pairs$block_id == block]
    runit.text <- paste("(", conc.runit, ")", sep = "")
    cunit.text <- paste("(", conc.cunit, ")", sep = "")
    drug.row <- drug.pairs$drug_row[drug.pairs$block_id == block]
    drug.col <- drug.pairs$drug_col[drug.pairs$block_id == block]

    plot.title <- paste("Dose-response matrix (inhibition)", "\nBlockID:",
                          block, sep = " ")

    # plot heatmap for dose-response matrix
    axis.x.text <- signif(as.numeric(levels(data.plot$conc_c)), 4)
    axis.y.text <- signif(as.numeric(levels(data.plot$conc_r)), 4)
    dose.response.p <- ggplot2::ggplot(data = data.plot, 
                                    aes(x=conc_c, y=conc_r, fill=Inhibition)) +
      ggplot2::geom_tile() +
      ggplot2::geom_text(ggplot2::aes(label = Inhibition)) +
      ggplot2::scale_fill_gradient2(low = "green", high = "red",
                                    midpoint = 0, name = "Inhibition (%)") +
      ggplot2::scale_x_discrete(labels = axis.x.text) +
      ggplot2::scale_y_discrete(labels = axis.y.text) +
      ggplot2::xlab(paste(drug.col, cunit.text, sep = " ")) +
      ggplot2::ylab(paste(drug.row, runit.text, sep = " "))
    
    # Add the title for heatmap
    dose.response.p <- dose.response.p + 
      ggplot2::ggtitle(plot.title) + 
      ggplot2::theme(plot.title = ggplot2::element_text(size = 20))
    
    # Set label's style of heatmap
    dose.response.p <- dose.response.p + 
      ggplot2::theme(axis.text.x = ggplot2::element_text(color = "red", 
                                                         face = "bold", 
                                                         size = 15),
                     axis.text.y = ggplot2::element_text(color = "red",
                                                         face = "bold",
                                                         size = 15),
                     axis.title = ggplot2::element_text(size = 15))
    
     
    # Fit model for the row drug
    drug.row.response <- ExtractSingleDrug(response.mat, dim = "row")
    drug.row.model <- FitDoseResponse(drug.row.response)#, ...)

    graphics::layout(matrix(c(1, 3, 2, 3), 2, 2, byrow = TRUE))
    # plot the curve for the row drug
    suppressWarnings(graphics::par(mgp=c(3, .5, 0)))
    x.lab <- paste("Concentration", runit.text, sep = " ")
    graphics::plot(drug.row.model, xlab = x.lab, ylab = "Inhibition (%)",
                   type = "obs", col = "red", cex = 1.5, pch = 16)#, ...)
    graphics::plot(drug.row.model, xlab = x.lab, ylab = "Inhibition (%)", 
                   type = "none", cex = 1.5, add = TRUE, lwd = 3)
    graphics::title(paste("Dose-response curve for drug:", drug.row, "in Block", 
                         block), cex.main = 1)

    # Fit model for the col drug
    drug.col.response <- ExtractSingleDrug(response.mat, dim = "col")
    drug.col.model <- FitDoseResponse(drug.col.response)#, ...)

    # plot the curve for the col drug
    x.lab <- paste("Concentration", cunit.text, sep = " ")
    graphics::plot(drug.col.model, xlab = x.lab, ylab = "Inhibition (%)",
                   type = "obs", col = "red", cex = 1.5, pch = 16)#, ...)
    graphics::plot(drug.col.model, xlab = x.lab, ylab = "Inhibition (%)",
                   type = "none", cex = 1.5, add = TRUE, lwd = 3)
    graphics::title(paste("Dose-response curve for drug:", drug.col, "in Block",
                          block), cex.main = 1)

    graphics::plot.new()
    #vps <- baseViewports()
    #pushViewport()
    print(dose.response.p, vp = grid::viewport(height = grid::unit(1, "npc"), 
                                               width = grid::unit(0.5, "npc"), 
                                               just = c("left","top"),
                                               y = 1, x = 0.5))
    #popViewport()
    merge.plot <- grDevices::recordPlot()
    plots[[block]] <- merge.plot
    if(save.file) {
      if(is.null(file.name)){
        if (adjusted) {
          file <- paste(drug.row, drug.col, "adjusted_dose_response_block",
                            block, sep = "_")
        } else {
          file <- paste(drug.row, drug.col, "original_dose_response_block",
                            block, sep = "_")
        }
      } else {
        file <- file.name[i]
        i <- i + 1
      }
      
      if (!file.type %in% c("jpeg", "bmp", "png", "tiff", "pdf")){
        warning("Can not save plot in ", file.type, " format. Avaliable formats
                are 'jpeg', 'bmp', 'png', 'tiff',and 'pdf'.")
      } else if (file.type  == "pdf") {
        grDevices::pdf(paste(file, file.type, sep="."), width=12, height=6)
      } else {
        do.call(file.type, args=list(filename=paste(file, file.type, sep="."),
                                     width = 12, height = 6, units = "in",
                                     res = 600))
      }
      #
      #grDevices::jpeg(file.name, width = 800, height = 600)
      grDevices::replayPlot(merge.plot)
      grDevices::dev.off()
    } else {
      grDevices::dev.new(noRStudioGD = TRUE)
      grDevices::replayPlot(plots[[block]])
      grDevices::dev.off()
    }
  }
 return(NULL)
}
