# SynergyFinder
# Functions for fitting single drug dose-response curve
#
# Functions on this page:
# FitDoseResponse: Fitting single drug dose-response model
# FindModelType: Get the model type which was used to fit dose-response curve.


#' Fitting single drug dose-response model
#'
#' Function \code{FitDoseResponse} fits dose-response model by using
#' \code{\link[drc]{drm}} function.
#'
#' Pre-fitting process:
#' 1. Change the 0 value in concentration into 10^-10 to avoide raising error
#' when taking log.
#' 2. If the variance of "response" values equal to 0, add 10^-10 to the last
#' "response" value.
#'
#' Model choice:
#' First use "L.4" model to fit the raw data. If error or waring occurs, use
#' "LL.4" model to fit \code{log(raw data)}.
#'
#' @param data A data frame. It contains two columns:
#' \itemize{
#'   \item \strong{conc} The concentration of drugs added in experiment.
#'   \item \strong{response} The response of cell lines to drug with different
#'   concentrations.
#' }
#' @param Emin A numeric or \code{NA}. the minimal effect of the drug used in
#'    the 4-parameter log-logistic function to fit the dose-response curve. If
#'    it is not NA, it is fixed the value assigned by the user. Default setting
#'    is \code{NA}.
#' @param Emax A numeric or \code{NA}. the maximal effect of the drug used in
#'    the 4-parameter log-logistic function to fit the dose-response curve. If
#'    it is not NA, it is fixed the value assigned by the user. Default setting
#'    is \code{NA}.
#'
#' @return An object of class 'drc'. It contains imformation of fitted model.
#'
#' @author \itemize{
#'    \item{Liye He \email{liye.he@helsinki.fi}}
#'    \item{Shuyu Zheng \email{shuyu.zheng@helsinki.fi}}
#' }
#'
#' @references Seber, G. A. F. and Wild, C. J (1989)
#' href{https://onlinelibrary.wiley.com/doi/book/10.1002/0471725315}{Nonlinear
#' Regression, New York}: Wiley \& Sons (p. 330).
#'
#' @export
#' 
#' @examples
#' df <- data.frame(response = c(0, 29, 59, 60, 75, 90),
#'                  dose = c(0.00, 9.7656, 39.0626, 156.25, 625, 2500))
#' model <- FitDoseResponse(df)
FitDoseResponse <- function(data, Emin = NA, Emax = NA) {
  if (!all(c("dose", "response") %in% colnames(data))) {
    stop("The input must contain columns: \"dose\", \"respone\".")
  }
  
  if (nrow(data) != 1 & stats::var(data$response) == 0) {
    data$response[nrow(data)] <- data$response[nrow(data)] + 
      10^-10
  }
  
  drug.model <- tryCatch({
    drc::drm(response ~ dose, data = data, fct = drc::LL.4(fixed = c(NA, 
             Emin = Emin, Emax = Emax, NA)), na.action = stats::na.omit, 
             control = drc::drmc(errorm = FALSE, noMessage = TRUE, 
                                 otrace = TRUE))
    # }, warning = function(w) {
    #   print(w)
    #   data$dose[which(data$dose == 0)] <- 10^-10
    #   drc::drm(response ~ log(dose), data = data, fct = drc::L.4(fixed = c(NA,
    #            Emin = Emin, Emax = Emax, NA)), na.action = stats::na.omit,
    #            control = drc::drmc(errorm = FALSE, noMessage = TRUE,
    #                                otrace = FALSE))
  }, error = function(e) {
    print(e)
    data$dose[which(data$dose == 0)] <- 10^-10
    drc::drm(response ~ log(dose), data = data, fct = drc::L.4(fixed = c(NA, 
             Emin = Emin, Emax = Emax, NA)), na.action = stats::na.omit, 
             control = drc::drmc(errorm = FALSE, noMessage = TRUE, 
                                 otrace = FALSE))
  })
  return(drug.model)
}


#' Find the type of model used for fitting dose response data
#'
#' \code{FindModelType} will extract the model type ("LL.4" or "L.4) eventually 
#' used in funciton \code{\link{FitDoseResponse}} 
#' 
#' @param model An object of class 'drc'. It is generated by function 
#' \code{\link{FitDoseResponse}}
#'
#' @return A character either "LL.4" or "L.4". It indicates the type of 
#' model used for fitting dose response data.
#' 
#' @author Shuyu Zheng \email{shuyu.zheng@helsinki.fi}
#' @export
#'
#' @examples
#' df <- data.frame(response = c(0, 29, 59, 60, 75, 90),
#'                  dose = c(0.00, 9.7656, 39.0626, 156.25, 625, 2500))
#' model <- FitDoseResponse(df)
#' model.type <- FindModelType(model)
FindModelType <- function(model) {
  type <- model$call$fct[[1]][[3]]
  return(type)
}